package mercury;

import java.net.*;
import java.io.*;
import java.util.*;
import mercury.exception.*;
import mercury.util.*;

/**
 * <code>Server</code> is the standalone Xander server that synchronizes with other
 * Xander servers to ensure fail over. Each server has a current cluster status (CCS) that
 * is synchronized with all other Xander servers in the cluster. <br>
 * 
 * last modified - 21 March 2001<br>
 * version 1.0<br>
 * author Chang Sau Sheong
 *
 */

public class Server {
	
	Config config = Config.getInstance();
	boolean debug = new Boolean(config.getProperty("debug")).booleanValue();

	private static int _DEFAULT_PORT	= 5055;
	private static String _DEFAULT_SERVER = "localhost";
	private static int _SOCKET_BUFFER = 100;
	private static boolean _isPrimary = false;
	protected static boolean declared = false;
	
	/**
	 main() method
	*/
	
	public static void main(String[] args) throws IOException {
		// use the current cluster status
		Status status = Status.getInstance();
		Config config = Config.getInstance();
		
		// initialize starting values		
		String server_ip = getHost(args);
		int server_port = getPort(args);
		_isPrimary = checkIfPrimary(args);
		
		status.setRunningServer(server_ip);
		status.setRunningPort(server_port);
		
		if (_isPrimary) {
			status.promote();
		}
		status.addServer(server_ip, server_port);
		
		System.out.println("Started Mercury Server 1.0 at " + status.getRunningServer() + " : " + status.getPort(status.getRunningServer()));

		// create server-side TCP socket to listen to port
		ServerSocket serverSocket = null;
   	boolean listening = true;

		// create server Socket
		try {
	 		serverSocket = new ServerSocket(status.getPort(status.getRunningServer()),
	 																		 _SOCKET_BUFFER, 
	 																		 InetAddress.getByName(status.getRunningServer()));
   	}
		catch (Exception e) {
			System.err.println("### Error : could not get a connection on the socket. Server cannot be started.");
			e.printStackTrace();
			System.exit(-1);
		}   	

   	// Create the synchronizer to synchronize between servers
		Synchronizer synchronizer = new Synchronizer();
   	String appClassName = config.getProperty("xander.app.classname");
   	Application app = new Scheduler();
   	try {
   		app = (Application)Class.forName(appClassName).newInstance();   		
   	}
   	catch (ClassNotFoundException e) {
   		System.err.println("### Class not found. Please check your configuration file.");
   		e.printStackTrace();
   		System.exit(-1);
   	}
   	catch (InstantiationException e) {
   		System.err.println("### Cannot instantiate " + appClassName + ". Please check your configuration file. Your class could be abstract or an interface.");
   		System.exit(-1);
   	}
   	catch (Exception e) {
   		System.err.println("### Error : cannot create application. Please check your configuration file.");
   		e.printStackTrace();
   		System.exit(-1);
   	}

		SetupServerActions setup = new SetupServerActions();
		setup.execute();

    // listen on sockets from incoming connections from clients  	
  	while (listening) {  	
    	synchronizer.sync();
   		(new MultiServerThread(serverSocket.accept())).start();
			if (status.isPrimary()) {
				System.out.println("*-- primary --*");
				app.activate();
			}
    }

		// close server socket
   	serverSocket.close();
	}


	/**
	 Get the hostname from the command line
	*/

  private static String getHost(String args[]) {
    String defHost = _DEFAULT_SERVER;  /* Default host name  */
    int length = args.length;

    for (int i = 0; i < length; i++) {
    	if (args[i].equals("-server") || args[i].equals("-s")) {
      	if (++i < length) {
        	return(args[i]);
        }
      }
		}
		return(defHost);
	}

	/**
	 Get the port from the command line
	*/

  private static int getPort(String args[]) {
      int defPort = _DEFAULT_PORT;   /* Default port number  */
      int length = args.length;
	    for (int i = 0; i < length; i++) {
	      if (args[i].equals("-port") || args[i].equals("-p")) {
	  	    if (++i < length) {
	        	return(Integer.parseInt(args[i]));
	        }
	      }    
     	}
      return(defPort);
  }

	/**
	 Check if this is started up as the primary server
	*/

  private static boolean checkIfPrimary(String args[]) {
      int length = args.length;
	    for (int i = 0; i < length; i++) {
       if (args[i].equals("-primary") || args[i].equals("-P")) {
       	return (true);
        }    
      }
      return(false);
  }

}
