package mercury;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.Vector;
import java.util.Iterator;
import java.util.List;
import java.util.Hashtable;
import java.util.Enumeration;
import java.net.*;


/**
 * <code>Status</code> represents the current cluster status of the servers. It is
 * a trasient object, meaning that the state of this object is not written to any persistent
 * storage. This is because the state of the server is retrieved from the current primary server.
 * <p>
 * last modified - 19 March 2001<br>
 * version 1.0<br>
 * author Chang Sau Sheong
 *
 */

public class Status {

	private boolean _primaryFlag = false;
	private Hashtable _servers = new Hashtable();
	private String _primaryServer = new String();
	private String _runningServer = new String();
	private int _runningPort = 0;

	private static Status _instance;
	
  /**
  Gets a list of servers
  @returns Hashtable Hashtable containing a list of server IP address/port pairs
  */
	public Hashtable getServers() {
		return _servers;
	}

	/**
	Gets a list of servers and ports in a semi-colon delimited string
	@returns String A list of servers and ports in a semi-colon (;) delimited string
	*/
	
	public String getServersAsString() {
	
		StringBuffer servers = new StringBuffer();		
		Enumeration serversEnum = _servers.keys();
		
		while (serversEnum.hasMoreElements()) {
			String serverName = (String)serversEnum.nextElement();
			servers.append( serverName + ":" + _servers.get(serverName).toString() + ";");
		}
		return servers.toString();
		
	}
	
	/**
	Set the servers list in the current cluster status
	@parameter A hashtable containing a list of server IP addresses/ports pair
	*/
	
	public void setServers(Hashtable servers) {
		_servers = servers;
	}
	
	/**
	Gets the primary NotifyServer IP address
	*/
	public String getPrimaryServer() {
		return _primaryServer;
	}

	/**
	Makes this the Primary Notify Server
	*/
	public void promote() {
		_primaryFlag = true;
	}

	public void demote() {
		_primaryFlag = false;
	}
	

  /**
  Gets the port corresponding to the IP address. Returns a null if there is no such
  server.
  */	
	public int getPort(String ip) {
		return ((Integer)_servers.get(ip)).intValue();	
	}
	
	/**
	Check if this server is the Primary Notify Server (PNS). Returns true if it is, false if it
	is not.
	*/
	public boolean isPrimary() {
		return _primaryFlag;
	}
	
	/**
	Adds a Notify Server to the list
	*/
	public void addServer(String ip, int port) {
		_servers.put(ip, new Integer(port));
	}
	
	/**
	Remove a server from the list
	*/
	public void removeServer(String ip) {
		_servers.remove(ip);	
	}
	
	/**
	Sets the currently running server IP. This could be different from the localhost IP
	if there are more than 1 IP addresses in the same machine
	*/
	public void setRunningServer(String ip) {
		_runningServer = ip;
	}

	/**
	Gets the currently running server IP
	*/
	public String getRunningServer() {
		return _runningServer;
	}

	/**
	Sets the currently running server port. This could be different from the localhost port
	if there are more than 1 IP addresses in the same machine
	*/
	public void setRunningPort(int port) {
		_runningPort = port;
	}

	/**
	Gets the currently running server port
	*/
	public int getRunningPort() {
		return _runningPort;
	}

	
  /**
  Creates the status class to store the Xander server status.
  */
  private Status() {    
    try {
	    _runningServer = InetAddress.getLocalHost().getHostAddress();
			_servers.clear();
    }
    catch (Exception e) {
    	System.err.println("## Cannot create a status object." + e.getMessage());
    }
  }

	/**
	Get the singleton instance
	*/
  public static synchronized Status getInstance() {
    if (_instance == null) {
      _instance = new Status();
    }
    return _instance;
  }
	

}